#ifndef BufferedSoftwareSerial_h_
#define BufferedSoftwareSerial_h_

// Anbindung des Bluetooth-Moduls (HC-05) über Software-Serial() mit Sende-Puffer:
//
#include <SoftwareSerial.h>
//
// SoftwareSerial hat keine TX-Puffer. Dadurch blockieren längere Sendungen
//  den gesamten Programmablauf.
// BufferedSoftwareSerial implementiert einen TX-Puffer, in den die benötigten
//  print() Funktionen ihre Ausgabe schreiben. Das Senden muss dann in loop()
//  Zeichen für Zeichen mit SendNextChar() veranlasst werden. Je nach Dauer
//  eines loop() Durchlaufs sinkt die effektive Baudrate dann massiv ab.
//
class BufferedSoftwareSerial : protected SoftwareSerial {
  public:
    BufferedSoftwareSerial(int rx, int tx) : SoftwareSerial(rx, tx)
    {
      iWrite = 0;
      iSend  = 0;
    }

    bool    SendNextChar();
    void    println()		  {write(10);}
    void    print(const char* s);
    void    print(int i);
    void    print(unsigned int i);
    void    print(long l);
    void    print(unsigned long l);
    void    println(const char* s)  {print(s); println();}
    void    println(int i)	    {print(i); println();}
    void    println(unsigned int i) {print(i); println();}
    void    println(long l)	    {print(l); println();}
    void    println(unsigned long l){print(l); println();}
    bool    write(int8_t i);

    // von außen zugängliche Funktionen der Basisklasse:
    //
    using SoftwareSerial::read;
    using SoftwareSerial::available;
    using SoftwareSerial::begin;

  protected:
    bool    IsBufferEmpty() const     {return iWrite == iSend;}

  protected:
    enum {bufferSize = 200};
    int8_t  buffer[bufferSize];
    int	    iWrite;
    int	    iSend;
};


//
// Nächstes gepuffertes Zeichen senden.
// 
// Rückgabe: false wenn Puffer leer war.
//
inline
bool BufferedSoftwareSerial::SendNextChar()
{
  if (IsBufferEmpty()) {
    return false;
  }
  SoftwareSerial::write(buffer[iSend]);	  // nächstes Zeichen mit write() der Basisklasse abschicken
  iSend++;
  if (iSend >= bufferSize) {
    iSend = 0;
  }
  return true;	
}


//
// Zeichen in den Puffer schreiben. Überschreibt/ersetzt SoftwareSerial::write(int8_t i)
//
// Rückgabe: false wenn der Puffer überläuft.
//
inline
bool BufferedSoftwareSerial::write(int8_t i)
{
  buffer[iWrite] = i;
  iWrite++;
  if (iWrite >= bufferSize) {
    iWrite = 0;
  }
  if (iWrite == iSend) {
    if (iWrite == 0) {
      iWrite = bufferSize;
    }
    iWrite--;
    return false;
  }
  return true;
}


//
// String in den Puffer schreiben
//
inline
void BufferedSoftwareSerial::print(const char* s)
{
  char c;
  while ((c = *s) != 0) {write(c); s++;}
}


#endif
