//#if defined(EOS1100D) || defined(EOS1200D_1300D) || defined(EOS2000D)

// P2 = GND (Return GND for phototransistor/diode)

// Zur Erkennung des LV wird geprft, ob die Coil Haltedauer in einem bestimmten Limit liegt.
// Dieses Limit wird aber auch fr sehr kurze Belichtungen unterschritten (1100D: t <= 1/500s,
//  1200D: t <= 1/200s), was dann zu einem Err30 fhrt!
// Zwar sind solche Zeiten nicht sinnvoll, aber da die Kameras auch ohne Erkennung des LV
//  alle Funktionen korrekt ausfhren, kann auf die Erkennung auch verzichtet werden:
//
#define OHNE_LV_ERKENNUNG

#if defined(OHNE_LV_ERKENNUNG)
# define COIL_HOLD_ENTER_LV  90	  // wird auch von krzesten Bel. nicht unterschritten
#else
# if defined(EOS1100D)
#  define COIL_HOLD_ENTER_LV  103
# elif defined(EOS2000D)
#  define COIL_HOLD_ENTER_LV  108  // 108.5
# else
#  define COIL_HOLD_ENTER_LV  107
# endif
#endif


// Die 1100D etc. haben nur eine Coil und einen Vorhang, der beim normalen Auslsen geschlossen,
//  im LiveView geffnet ist. Beim Auslsen luft der Motor an und ffnet, gleichzeitig
//  wird die Coil aktiviert. Wenn der Curtain geffnet ist, erzeugt die Lichtschranke (OPT1)
//  eine Lo/Hi/Lo Folge und der Motor stoppt mit ROT3=LOW. Bis hier ist das Timing identisch
//  fr Short, Long und Bulb.
//  Die interne Feder ist jetzt aufgezogen, so dass der Vorhang schliet, sobald die Coil lst.
//  Die Aufnahme beginnt wohl elektronisch durch Beenden der Sensor-Lschfunktion, ausgelst
//  durch OPT1 (oder ROT3). Sie endet durch Lsen der Coil. Beim Schlieen erzeugt OPT1 (nach
//  ca. 5 ms) die umgekehrt Lo/Hi/Lo Folge. Abschlieend dreht der Motor wieder bis zur 
//  Startposition mit ROT1=LOW (kurz vor ffnung).
// Der LV startet etwas langsamer und stoppt schon kurz bevor ROT2 Hi (und ROT3 Lo) wird.
//
// Strategie: 
//  - Wenn die c1_L-Dauer bei COIL_HOLD_ENTER_LV ms liegt, ist der LV-Mode gestartet. Dann wird
//    mit c1_H_EnterLV abgeschlossen und fr das nchste c1_L c1_L_StartInLV verwendet.
//  - Wenn die c1_L-Dauer bei 48 ms liegt, ist der LV-Mode verlassen. Dann wird
//    mit c1_H_LeaveLV abgeschlossen und fr das nchste c1_L c1_L_Start verwendet.
//  
#define LoLIMIT_C1_ENTER_LV	((COIL_HOLD_ENTER_LV-3) ms)
#define HiLIMIT_C1_ENTER_LV	((COIL_HOLD_ENTER_LV+4) ms)
#define HiLIMIT_C1_LEAVE_LV	((48+5) ms)			// C1 braucht bei LeaveLV 48 ms 

#define IsInWindow_EnterLV(dt)  (dt <= HiLIMIT_C1_ENTER_LV  &&  dt >= LoLIMIT_C1_ENTER_LV)
#define IsInWindow_ReEnterLV(dt) IsInWindow_EnterLV(dt)
#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C1_LEAVE_LV)

#ifdef EOS1100D
sAction c1_L_Start[] = {			// Coil aktiv
	{25850L,  ROT1, HIGH }, // 0
	{34 ms,   OPT1, LOW  }, // 1
	{35100L,  ROT2, LOW  }, // 2
	{39400L,  OPT1, HIGH }, // 3
	{42 ms,   OPT1, LOW  }, // 4
	{62900L,  ROT2, HIGH }, // 5
	{71400L,  ROT3, LOW  }, // 6
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};

// Fortsetzung des regulren Shoot-Timings nach Coil Hi
//
sAction c1_H_FinishShoot[] = {
	{4 ms,  OPT1, HIGH },	      // Achtung: 5.8 ms fhrt im LV (ohne Erkennung) z Err30!
	{6 ms,  OPT1, LOW  }, // 8
	{7 ms,  OPT1, HIGH }, // 9
	{34800L, ROT3, HIGH }, // 10
	{71 ms,  ROT4, LOW  }, // 11
	{107 ms,  ROT4, HIGH }, // 12
	{108 ms,  ROT1, LOW  }, // 13
	STOP_ACTION,
};

// Fortsetzung des LV_Shoot-Timings nach Coil Hi
// C1_H kommt im LV schon nach 66 ms. Das ist noch vor ROT3_L, aber nach ROT2_H, das
//  im LV auch frher kommt.
//
sAction c1_H_FinishLVShoot[] = {
	{0 ms,  ROT2, HIGH }, // sollte schon erledigt sein
	{0 ms,  ROT3, LOW  }, // fehlt noch, wenn mit c1_L_Start gestartet
	{4 ms, OPT1, HIGH },	
	{6 ms,   OPT1, LOW },	
	{7 ms,   OPT1, HIGH },	
	{36 ms, ROT3, HIGH }, // 10
	{75 ms,  ROT4, LOW  }, // 11
	{112 ms,  ROT4, HIGH }, // 12
	{112 ms,  ROT1, LOW  }, // 13
	STOP_ACTION,
};


// Im LV kommt C1_L entweder fr Shoot, Re-Enter oder Leave,
// Nach einem c1_H_EnterLV folgt immer c1_L_StartInLV, entweder fr Shoot oder Leave,
//  die mit dem gleichen timing beginnen:
//
sAction c1_L_StartInLV[] = {			// Coil aktiv
	{27 ms,  ROT2, HIGH },
	{37 ms,  ROT3, LOW  },
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};


// Nach c1_H_FinishLVShoot (das an t=66ms erkannt wird) kommt eine Re-EnterLV Sequenz
//
//
sAction c1_H_EnterLV[] = {
	{0 ms,   ROT2, LOW },  	//	korrigieren: wurde in c1_L_Start schon gesetzt
	{0 ms,   ROT3, HIGH  },	//	korrigieren: wurde in c1_L_Start schon gesetzt
	STOP_ACTION,
};


// LV beenden (klappt auch mit c1_H_FinishShoot)
// Coil 1 lst im LV schon nach 48 ms:
//  c1_L_Start ist bis Schritt 5 ausgefhrt,
//
sAction c1_H_LeaveLV[] = {
	{ 0 ms,  OPT1, LOW  }, // sollte schon anliegen
	{ 4 ms,  OPT1, HIGH },
	{5500L,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 22 ms, ROT3, HIGH },
	{ 52 ms, ROT4, LOW  },
	{ 89 ms, ROT4, HIGH },
	{ 90 ms, ROT1, LOW  },
	STOP_ACTION,
};

#elif defined(EOS1200D_1300D)

// Die 1200D hat einen sehr hnlichen Shutter wie die 1100D mit minimal anderem Timing.
// (Beide Kameras akzeptieren beide Timings!)
// Die 1200D fhrt im LV den Motor offenbar etwas zurck und erzeugt dabei
//  zustzliche Signale auf R2, die aber nur fr LV-Start abgeprft werden.
//
sAction c1_L_Start[] = {			// Coil aktiv
	{24 ms,   ROT1, HIGH }, // 0
	{34 ms,   OPT1, LOW  }, // 1
	{36 ms,   ROT2, LOW  }, // 2
	{39 ms,   OPT1, HIGH }, // 3
	{41 ms,   OPT1, LOW  }, // 4
	{56 ms,   ROT2, HIGH }, // 5
	{65 ms,   ROT3, LOW  }, // 6
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};

// Fortsetzung des regulren Shoot-Timings nach Coil Hi
//
sAction c1_H_FinishShoot[] = {
	{4 ms,  OPT1, HIGH },	      // Achtung: 5.8 ms fhrt im LV (ohne Erkennung) zu Err30!
	{6 ms,  OPT1, LOW  }, 
	{7 ms,  OPT1, HIGH }, 
	{35 ms, ROT3, HIGH }, 
	{70 ms,  ROT4, LOW  }, 
	{105 ms, ROT4, HIGH }, 
	{105 ms, ROT1, LOW  }, 
	STOP_ACTION,
};

// Fortsetzung des LV_Shoot-Timings nach Coil Hi
// C1_H kommt im LV schon nach 66 ms. Das ist noch vor ROT3_L, aber nach ROT2_H, das
//  im LV auch frher kommt.
//
sAction c1_H_FinishLVShoot[] = {
	{4 ms,   OPT1, HIGH },	
	{6 ms,   OPT1, LOW },	
	{7 ms,   OPT1, HIGH },	
	{37 ms,  ROT3, HIGH }, // 10
	{71 ms,  ROT4, LOW  }, // 11
	{107 ms, ROT4, HIGH }, // 12
	{108 ms, ROT1, LOW  }, // 13
	STOP_ACTION,
};


// Im LV kommt C1_L entweder fr Shoot, Re-Enter oder Leave,
// Nach einem c1_H_EnterLV folgt immer c1_L_StartInLV, entweder fr Shoot oder Leave,
//  die mit dem gleichen timing beginnen:
//
sAction c1_L_StartInLV[] = {			// Coil aktiv
	{28 ms,  ROT2, HIGH },
	{38 ms,  ROT3, LOW  },
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};


//
// Nach c1_H_FinishLVShoot kommt eine Re-EnterLV Sequenz
//
sAction c1_H_EnterLV[] = {
	{0 ms,   ROT2, LOW },  	//	korrigieren: wurde in c1_L_Start schon gesetzt
	{0 ms,   ROT3, HIGH  },	//	korrigieren: wurde in c1_L_Start schon gesetzt
	STOP_ACTION,
};


// LV beenden (klappt auch mit c1_H_FinishShoot)
// Coil 1 lst im LV schon nach 48 ms:
//  c1_L_Start ist bis Schritt 5 ausgefhrt,
//
sAction c1_H_LeaveLV[] = {
	{ 0 ms,  OPT1, LOW  }, // sollte schon anliegen
	{ 4 ms,  OPT1, HIGH },
	{ 6 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 18 ms, ROT3, HIGH },
	{ 49 ms, ROT4, LOW  },
	{ 83 ms, ROT4, HIGH },
	{ 84 ms, ROT1, LOW  },
	STOP_ACTION,
};

#else // EOS2000D:

// Die 2000D hat einen sehr hnlichen Shutter wie ihre Vorgnger mit minimal anderem Timing.
// Wie die 1200D fhrt sie im LV den Motor offenbar etwas zurck und erzeugt dabei
//  zustzliche Signale auf R2, die aber nur fr den LV-Start abgeprft werden!
//
sAction c1_L_Start[] = {			// Coil aktiv
//*
	{24 ms,   ROT1, HIGH }, // 0
	{32 ms,   OPT1, LOW  }, // 1
	{34 ms,   ROT2, LOW  }, // 2
	{36 ms,   OPT1, HIGH }, // 3
	{40 ms,   OPT1, LOW  }, // 4
	{56 ms,   ROT2, HIGH }, // 5
	{54 ms,   ROT2, LOW  }, // hier nicht bentigt, aber fr LV
	{61 ms,   ROT2, HIGH }, //  Korrektur
	{62 ms,   ROT3, LOW  }, // 6
/*/ // Timing wie fr LV Start: Ist auch fr Shoot ok!
  	{40 ms,   ROT1, HIGH }, // 0
	{52 ms,   OPT1, LOW  }, // 1
	{57 ms,   OPT1, HIGH }, // 3
	{59 ms,   ROT2, LOW  }, // 2
	{60 ms,   OPT1, LOW  }, // 4
	{61 ms,   ROT2, HIGH }, // 5
	{64 ms,   ROT2, LOW  }, // hier nicht bentigt, aber fr LV
	{67 ms,   ROT2, HIGH }, //  Korrektur: wird aber nicht bentigt
	{67 ms,   ROT3, LOW  }, // 6
//*/	
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};

// Fortsetzung des regulren Shoot-Timings nach Coil Hi
//
sAction c1_H_FinishShoot[] = {
	{4 ms,  OPT1, HIGH },
	{6 ms,  OPT1, LOW  }, 
	{7 ms,  OPT1, HIGH }, 
	{33 ms, ROT3, HIGH }, 
	{56 ms,  ROT4, LOW  }, 
	{98 ms, ROT4, HIGH }, 
	{99 ms, ROT1, LOW  }, 
	STOP_ACTION,
};

// Fortsetzung des LV_Shoot-Timings nach Coil Hi
// C1_H kommt im LV schon nach 66 ms. Das ist noch vor ROT3_L, aber nach ROT2_H, das
//  im LV auch frher kommt.
//
sAction c1_H_FinishLVShoot[] = {
	{4 ms,   OPT1, HIGH },	
	{6 ms,   OPT1, LOW },	
	{7 ms,   OPT1, HIGH },	
	{34 ms,  ROT3, HIGH }, // 10
	{66 ms,  ROT4, LOW  }, // 11
	{100 ms, ROT4, HIGH }, // 12
	{100 ms, ROT1, LOW  }, // 13
	STOP_ACTION,
};


// Im LV kommt C1_L entweder fr Shoot, Re-Enter oder Leave,
// Nach einem c1_H_EnterLV folgt immer c1_L_StartInLV, entweder fr Shoot oder Leave,
//  die mit dem gleichen timing beginnen:
//
sAction c1_L_StartInLV[] = {			// Coil aktiv
	{26 ms,  ROT2, HIGH },
	{36 ms,  ROT3, LOW  },
	HALT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
};


//
// Nach c1_H_FinishLVShoot kommt eine Re-EnterLV Sequenz
//
sAction c1_H_EnterLV[] = {
	{0 ms,   ROT2, LOW },  	//	korrigieren: wurde in c1_L_Start schon gesetzt
	{0 ms,   ROT3, HIGH  },	//	korrigieren: wurde in c1_L_Start schon gesetzt
	STOP_ACTION,
};


// LV beenden (klappt auch mit c1_H_FinishShoot)
// Coil 1 lst im LV schon nach 48 ms:
//  c1_L_Start ist bis Schritt 5 ausgefhrt,
//
sAction c1_H_LeaveLV[] = {
	{ 0 ms,  OPT1, LOW  }, // sollte schon anliegen
	{ 4 ms,  OPT1, HIGH },
	{ 6 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 17 ms, ROT3, HIGH },
	{ 46 ms, ROT4, LOW  },
	{ 78 ms, ROT4, HIGH },
	{ 79 ms, ROT1, LOW  },
	STOP_ACTION,
};

#endif // EOS2000D
//#endif // EOS1100D || EOS1200D_1300D || EOS2000D ###################################
