//#ifdef EOS600D

/* Coil-Timing
	Modus	     Coil 1  Coil 2

	Shoot      76+t     76	      
	Enter LV    94      87
	Shoot LV   71+t     71
	Leave LV    40      --

*/
// Coil-2 Timing der 600D-3 
// (Die Blink-Werte des Main-13 in Klammern weichen stark davon ab! Vmtl. hat die
//	Start-Sequenz einen Einfluss auf das Timing.))
// - Das regulre C2_Hi kommt bei 77 ms (!81!), bei Shoot im LV nach 71(!74!) ms
// - Beim LV-Start kommt C2_Hi bei 85 ms, Re-Enter 88 ms (!68!).
//   (Dauer der 600D-3 Coil 1 Pulse: Enter LV: 91ms, Re-Enter LV 94ms)
// - Am Ende des LV bleibt C2 Hi
// --> Grenzwerte zur Unterscheidung der Funktion bei C1_H
#define HiLIMIT_C1_ENTER_LV	((83+2) ms)			// schneller als im Original, weil von C1_H in c1_L_Start ausgelst?
#define HiLIMIT_C1_REENTER_LV	((96+2) ms)
#define HiLIMIT_C1_LEAVE_LV	((41+4) ms)			// C1 braucht bei LeaveLV 41 ms 
//#define LoLIMIT_C2_ENTER_LV	((86-3) ms)			// aus 600D-3 Timing! Mit Main-13 68 ms: krzester Puls
#undef  HiLIMIT_C2_LEAVE_LV									// C2 wird fr LeaveLV gar nicht erst LOW
#define HiLIMIT_C2_LVSHOOT	((71+5) ms)			// groe Toleranz (+5) erfasst auch Main -13

#define IsInWindow_LVShoot(dt)   (dt < HiLIMIT_C2_LVSHOOT)	// 76
#define IsInWindow_EnterLV(dt)   (dt <= HiLIMIT_C1_ENTER_LV)	// 85
#define IsInWindow_ReEnterLV(dt) (dt <= HiLIMIT_C1_REENTER_LV)	// 98
#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C1_LEAVE_LV)	// 45


sAction c1_L_Start[] = {			// Coil aktiv
	{0 ms,   OPT1, LOW  }, // 0
	{0 ms,   OPT2, LOW  }, // 1
	{19 ms,  ROT1, HIGH }, // 2
	{30 ms,  ROT2, LOW  }, // 3
	{53 ms,  ROT2, HIGH }, // 4
	{62 ms,  ROT3, LOW  }, // ca. 12 ms spter als bei der 700D (14 ms vor C2_H bei 76 ms)
	//
	// Wenn nur C1 triggert, muss OPT2_Hi fr den LongMode hier schon gesetzt werden:
	{83 ms,  OPT2, HIGH }, // geht 7 ms nach C2_H Hi.
	#ifndef TRIGGER2_MODE
	//
	// OPT2 jetzt nochmals zu schalten behebt das Blende-Problem
	//  (das evtl. auch nur ohne Coil2 auftritt).
	// Ist bisher nur ohne Coil2 an der eo63a getestet.
	// (Evtl. erwartet die Kamera bei groen Blenden das OPT2 Hi spter!)
	//
	{93 ms,  OPT2, LOW }, // geht 7 ms nach C2_H Hi.
	{123 ms,  OPT2, HIGH }, // geht 7 ms nach C2_H Hi.
	#endif	  
	HALT_ACTION,
};


sAction c1_L_StartInLV[] = {			// Coil aktiv im LV
	{0 ms,   OPT1, LOW  }, // sollte schon anliegen
	//{0 ms,   OPT2, LOW  }, // entfllt
	//{19 ms,  ROT1, HIGH }, // sollte schon anliegen
	//{30 ms,  ROT2, LOW  }, // sollte schon anliegen
	{20 ms,  ROT2, HIGH }, // 4
	{31 ms,  ROT3, LOW  },
	{78 ms,  OPT2, HIGH }, // geht 7 ms nach C2_H Hi. Wenn nur C1 triggert, muss es fr den LongMode hier schon gesetzt werden.
	HALT_ACTION,
};


sAction c1_H_EnterLV[] = {			// Coil 1 lst im LV
	{0 ms,    ROT2, LOW }, //	korrigieren: wurde in c1_L_Start schon gesetzt
	{0 ms,    ROT3, HIGH  }, //	korrigieren: wurde in c1_L_Start schon gesetzt
	{7 ms,   OPT1, LOW},
	{7 ms,   OPT2, HIGH},	// es scheint egal zu sein welchen Pegel O1 und O2 hier bekommen!
	STOP_ACTION,
};

sAction c2_H_EnterLV[] = {			// identisch fr Coil 2
{0 ms,    ROT2, LOW }, //	korrigieren: wurde in c1_L_Start schon gesetzt
{0 ms,    ROT3, HIGH  }, //	korrigieren: wurde in c1_L_Start schon gesetzt
	{7 ms,   OPT1, LOW}, 	// bleibt LOW
	{7 ms,   OPT2, HIGH}, //	es scheint egal zu sein welchen Pegel O1 und O2 hier bekommen!
	STOP_ACTION,
};


// LV beenden (klappt auch mit c1_H_FinishShoot)
sAction c1_H_LeaveLV[] = {		// Coil 1 lst im LV:		
	{ 0 ms,  OPT2, HIGH }, // sollte schon anliegen
	{ 4 ms,  OPT1, HIGH }, // (Offset 41 abgezogen)
	{ 5 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 17 ms, ROT3, HIGH },
	{ 45 ms, ROT4, LOW  },
	{ 96 ms, ROT4, HIGH },
	{ 96 ms, ROT1, LOW  },
	STOP_ACTION,
};

// Fortsetzung des regulren Timings nach Coil-1/2 Hi
// Curtain von Coil 2 hat schon vorher geffnet und 7 ms danach ging OPT2 Hi - falls die Belichtung
//  > 7 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
//  mehr ausgefhrt, deshalb hier nachgeholt!
//
//#if defined(TRIGGER2_MODE)	// Mit Coil 2
sAction c2_H_FinishShoot[] = {			// Coil 2 wird inaktiv
	{6 ms,    OPT2, HIGH },	// -> Curtain ffnet und nach 6 ms geht OPT2 Hi. Wenn bei sehr kurzen 
													//			Belichtungen (<6ms, 1/200) C1 zuvor Hi wird, wird die Aktion vorgezogen.
													//		Tatschlich ist es auch ok, OPT2_H sofort nach C2_H auszufhren.
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
//#else
//sAction c1_H_FinishShoot[] = {
//#endif	
#define c1_H_FinishShoot (c2_H_FinishShoot + 2)	
	// Coil 1 wird inaktiv: fr Coil 1 allein kein LONGWAIT (mehr), weil bei HALT in c1_L_Start
	//	gewartet wurde
	// Curtain von Coil 2 hat schon vorher geffnet und 7 ms danach ging OPT2 Hi - falls die Belichtung
	//  > 7 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgefhrt, deshalb hier nachgeholt!
	{ 0 ms,  OPT2, HIGH },	// ohne Coil 2 bei kurzen Zeiten noch nicht ausgefhrt
	{ 4 ms,  OPT1, HIGH }, // (Offset 79 abgezogen)
	{ 5 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 26 ms, ROT3, HIGH },
	{ 57 ms, ROT4, LOW  },
	{106 ms, ROT4, HIGH },
	{107 ms, ROT1, LOW  },
	STOP_ACTION,
};


// ... im LV
//
//#if defined(TRIGGER2_MODE)	// Mit Coil 2 (Sichere Def fr C1 und C2, aber nicht parallel verfgbar:)
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
//{4 ms,    OPT2, HIGH },	// -> Curtain ffnet (OPT2 war gar nicht LOW)
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
//#else
//sAction c1_H_FinishLVShoot[] = {
//#endif	
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 1)	
	// Coil 1 wird inaktiv: fr Coil 1 allein kein LONGWAIT, weil bei HALT in c1_L_Start
	//	gewartet wurde
	// Curtain von Coil 2 hat schon vorher geffnet und 7 ms danach ging OPT2 Hi - falls die Belichtung
	//  > 7 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgefhrt, deshalb hier nachgeholt!
	{ 4 ms,  OPT1, HIGH }, // (Offset 79 abgezogen)
	{ 5 ms,  OPT1, LOW  },
	{ 7 ms,  OPT1, HIGH },
	{ 18 ms, ROT3, HIGH },
	{ 45 ms, ROT4, LOW  },
	{ 88 ms, ROT4, HIGH },
	{ 88 ms, ROT1, LOW  },
	STOP_ACTION,
};

// Nchste sAction Liste auswhlen fr die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
  // Coil-2 Timing der 600D-3 
  // (Die Blink-Werte des Main-13 in Klammern weichen stark davon ab! Vmtl. hat die
  //	Start-Sequenz einen Einfluss auf das Timing.))
  // - Das regulre C2_Hi kommt bei 77 ms (81), bei Shoot im LV nach 71(74) ms
  // - Beim LV-Start kommt C2_Hi bei 86 ms (68)
  // - Am Ende des LV bleibt C2 Hi
  //BlinkLED(dt);	// gibt auch im LV den Shoot-Wert
  //BlinkLEDForced(dt);	// gibt im LV den Re-Enter Wert
  if (dt < 70 ms) {					// statt dt>LoLIMIT_C2_ENTER_LV: C2_H kommt schon bei 68 ms, wenn EnterLV mit c1_L_Start eingeleitet wird!
    return c2_H_EnterLV;
  }
  else if (IsInWindow_LVShoot(dt)  	// (700D 50+9 ms, 600D 71+2 ms:) Auslsen im LV
	   &&  actionPtr != c1_L_Start	//20231213:  Hilft nich gegen das Blendenproblem! (6D: normales Shoot ausschlieen, das durch groe Blendenwerte auf hnliche Zeiten kommt)
  ) {
    return c2_H_FinishLVShoot;
  }
  else {																	// (700D 68 ms, 600D 77 ms:) nicht-LV-Shoot
    return c2_H_FinishShoot;
  }
}

//#endif // EOS600D --------------------------------------------
