/*////////////////////////////////////////////////////////////////////////
EOS650D_700D
		Startpunkt (stromlos stabil):
		 Innerer Curtain (iC) ist oben (= geschlossen), Anker an der unteren Coil (C2)
		 uerer Curtain (aC) ist oben (= geffnet), Anker an der oberen Coil (C1)

		Beim Auslsen dreht der Motor den aC ber den Ruhepunkt hinaus, er wird aber von C1 gehalten.
		Der Spiegel wird danach ausgelst (hat keine Coil!) und klappt hoch.
		Vmtl. dreht der Motor weiter bis der iC ebenfalls ausgelst, aber noch von C2 gehalten ist.
		C2 gibt dann iC frei, der (nach unten) ffnet, Belichtung beginnt.
		C1 gibt aC nach T frei, aC schliet (nach unten). Achtung: Der Auslser dafr ist nicht die
			Freigabe der C2, sondern das optische Signal beim ffnen des iC! D.h., bei der Emulation
			nur auf Basis von C1 muss OPT2_H nach der vorgegebenen Zeit generiert werden, sonst
			wird C1 nicht freigegeben!

		Abschlieend zieht der Motor den Spiegel nach unten und spannt ihn vor.
		Danach wird der iC hochgezogen (Anker bis C2) und zum Schluss der aC (Anker bis C1).

		Die Coil-Versorgung mit 4.7V liegt (anders als bei 600D und 1100D) permanent an und
		 knnte als Arduino-Versorgung dienen. Allerdings kommt sie wohl beim Start zu spt und
		 China-Pro_Mini brauchen mglicherweise auch zu viel Strom.

		Fr den Anschluss des Emulators kann das Kabel eines 1100D AF-Sensors (mit Testpunkten!)
		 verwendet werden.

		Regulrer Ablauf (nicht in LV) incl. Start und Restart des LV:
		------------------------------------------------------------------
		1.) Coil 1 --> LOW startet c1_L_Start. Stoppt nach 48 ms, oder nach:
		2.) Coil 2 --> HIGH: Kommt um die ffnungszeit vor Coil 1 HIGH
				a) Delay < 70ms	(= 68ms) startet c2_H_FinishShoot, das OPT setzt und dann auf Coil1 HIGH wartet.
				b) Delay >= 70ms (= 72ms) startet c2_H_EnterLV, das ein paar Signale korrigiert und stoppt.
		3.) Coil 1 --> HIGH: Sucht in der lfd. Tabelle LONGWAIT und geht weiter bzw. stoppt wenn b).

		Auslsung im LV:
		-----------------
		1.) Coil 1 --> LOW erkennt LV an der zuletzt durchlaufenen Tabelle (c2_H_FinishLVShoot oder c2_H_EnterLV)
				und startet c1_L_StartInLV.
		2.) Coil 2 --> HIGH: Kommt um die ffnungszeit vor Coil_1_HIGH oder zeitgleich dazu fr Leave LV.
				a) Delay < 46ms	(= 43ms, s.u.) startet c2_H_LeaveLV.
				b) Delay < 60ms (= 50ms, s.u.) startet c2_H_FinishLVShoot.
		3.) Coil 1 --> HIGH: Tut nix bei c2_H_LeaveLV, sonst wird LONGWAIT beendet (zuvor ggf. gesucht).
		4.) Coil 1 --> LOW bringt den Shutter (nach c2_H_FinishLVShoot) wieder in den LV-Zustand. Das weitere
				Timing ist fast identisch mit c2_H_EnterLV.

		Wird die Dauer der Pulse auf Coil 2 wird durch das Timing der ROT-Signale beeinflusst?


		// Komplettes Timing der 									".." = liegt schon an
		//  regulren Auslsung,									"x/y" = T / Bulb
		//  getriggert von Coil-1 LOW							"-" = entfllt
		sAction c1_L_Start[] = {	 // EnterLV	 Re-EnterLV	ShootLV		LeaveLV
			{0 ms,     OPT1, LOW  }, //		0					0					0				 -6 ???
			{0 ms,     OPT2, LOW  }, //		0					0					-					-
			{11 ms,    ROT1, HIGH }, //		12				12				..				..
			{12 ms,    ROTX, HIGH }, //		14				14				16				16
			{22 ms,    ROTX, LOW  }, //		27				30
			{32 ms,    ROT2, LOW  }, //		-					-					16				16
			{32 ms,    ROTX, HIGH }, //		-					-
			{48 ms,    ROT2, HIGH }, //		-					-					36/34			34
			{48 ms,    ROT3, LOW  }, //		-					-					36/35			34
			LONGWAIT_ACTION,
			// 68ms		 COIL2 HIGH					73				76				50				43				OPT1 wird hier Hi bis kurz vor C2_H wg. LED-Off
			//  +t     COIL1 HIGH					 +6.4			 +6.4			 +t				+0
			{ +6.4 ms, OPT2, HIGH }, //		 +6.4			 +6.4			..				..
			{ +t+4,    OPT1, HIGH }, //		 +6				 +6				+t+4			+4
			{ +t+5,    OPT1, LOW  }, //		-					-					+t+6			+6
			{ +t+7,    OPT1, HIGH }, //		-					-					+t+8			+7
			{ +t+18,   ROT3, HIGH }, //		-					-					+t+18			+16
			{ +t+47,   ROT4, LOW  }, //		-					-					+t+45			+43
			{ +t+91,   ROT4, HIGH }, //		-					-				+t+90/86		+82
			{ +t+91,   ROT1, LOW  }, //		-					-				+t+90/86		+82
			{ +t+111,  ROTX, LOW  }, //		-					-				+t+108/103	+98
			//																							+t+538 folgt C1_L fr Re-EnterLV
			STOP_ACTION,
		};

		////////////////////////////////////////////////////////////////////////////////////////*/

		// Dauer der Coil 1/2 Pulse in ms:
		// Enter LV: 79ms/73ms, Re-Enter LV 82ms/76ms
#define HiLIMIT_C1_ENTER_LV	((82+2) ms)
//#define HiLIMIT_C1_REENTER_LV	((82+2) ms)
#define LoLIMIT_C2_ENTER_LV	((73-3) ms)
#define HiLIMIT_C1_LEAVE_LV	((43+2) ms)			// C1 und C2 brauchen bei LeaveLV 43 ms 
#define HiLIMIT_C2_LEAVE_LV	((43+2) ms)			
#define HiLIMIT_C2_LVSHOOT	((50+9) ms)

#define IsInWindow_LeaveLV(dt)   (dt <= HiLIMIT_C1_LEAVE_LV)
#define IsInWindow_LVShoot(dt)   (dt < HiLIMIT_C2_LVSHOOT)
#define IsInWindow_EnterLV(dt)   (dt <= HiLIMIT_C1_ENTER_LV)
#define IsInWindow_ReEnterLV(dt) IsInWindow_EnterLV(dt)


// Sequenz fr regulres Auslsen bis Coil 2 abschaltet:
//
sAction c1_L_Start[] = {			// Coil aktiv
	{0 ms,   OPT1, LOW  }, // 0
	{0 ms,   OPT2, LOW  }, // 1
	{11 ms,  ROT1, HIGH }, // 2
	{12 ms,  ROTX, HIGH }, // 3
	{22 ms,  ROTX, LOW  }, // 4
	{32 ms,  ROT2, LOW  }, // 5
	{32 ms,  ROTX, HIGH }, // 6
	{48 ms,  ROT2, HIGH }, // 7
	{48 ms,  ROT3, LOW  }, // 8
	{79 ms,  OPT2, HIGH }, // OPT2 geht 6 ms nach C2_H Hi. Wenn nur C1 triggert, muss O2_H fr EnterLV (und fr den LongMode?) hier gesetzt werden.
			       //  Andernfalls wartet die Kamera mit dem C1_H (mehr als 300 ms) und streikt. 
			       //  Wenn O2_H zu frh kommt (75 ms) klappt EnterLV nicht.
	HALT_ACTION,
};

/* Die Sequenz fr den Start des LV (bis Coil 1 oder 2 Hi wird) hat durch das
//  Stoppen des Motors ein leicht abweichendes Timing. Zum Glck akzeptiert
// die 700D auch die deutlich abweichende Sequenz fr regulres Auslsen.
// (Der Fehler ohne C2 wird auch nicht dadurch verursacht.)
//
sAction c1_L_Start[] = {
	{0 ms,   OPT1, LOW  }, // 0
	{0 ms,   OPT2, LOW  }, // 1
	{12 ms,  ROT1, HIGH }, // 2
	{14 ms,  ROTX, HIGH }, // 3
	{27 ms,  ROTX, LOW  }, //
	{79 ms,  OPT2, HIGH },

	HALT_ACTION,
};
// Da Anfangs nicht erkannt werden kann, welche Aktion die Kamera vorhat,
//  wird stattdessen c1_L_Start ausgefhrt. Die 700D akzeptiert aber das
// Timing fr regulres Auslsen und strt sich auch nicht daran,
//  wenn die ROT-Signale wie beim regulren Auslsen folgen.
//*/


// Fortsetzung des regulren Timings nach Coil-1/2 Hi
//
//#if defined(TRIGGER2_MODE)	// Mit Coil 2   // Sichere Def fr C1 und C2, aber nicht parallel verfgbar
sAction c2_H_FinishShoot[] = {			// Coil 2 wird inaktiv
	{6 ms,    OPT2, HIGH },	// -> Curtain ffnet und nach 6 ms geht OPT2 Hi. Wenn bei sehr kurzen 
													//			Belichtungen (<6ms, 1/200) C1 zuvor Hi wird, wird die Aktion vorgezogen.
													//		Tatschlich ist es auch ok, OPT2_H sofort nach C2_H auszufhren.
	LONGWAIT_ACTION,				// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
//#else
//sAction c1_H_FinishShoot[] = {
//#endif	
#define c1_H_FinishShoot (c2_H_FinishShoot + 2)	
	// Coil 1 wird inaktiv: fr Coil 1 allein kein LONGWAIT (mehr), weil bei HALT in c1_L_Start
	//	gewartet wurde
	// Curtain von Coil 2 hat schon vorher geffnet und 7 ms danach ging OPT2 Hi - falls die Belichtung
	//  > 7 ms war. Bei sehr kurzen Belichtungen wird der letzte Eintrag in c1_L_Start aber nicht 
	//  mehr ausgefhrt, deshalb hier nachgeholt!
	{0 ms,    OPT2, HIGH },	// ohne Coil 2 bei kurzen Zeiten noch nicht ausgefhrt
	{4 ms,    OPT1, HIGH }, // 11 (Offset 78 abgezogen)
	{5 ms,    OPT1, LOW  }, // 12
	{7 ms,    OPT1, HIGH }, // 13
	{18 ms,   ROT3, HIGH }, // 14
	{47 ms,   ROT4, LOW  }, // 15
	{91 ms,   ROT4, HIGH }, // 16
	{91 ms,   ROT1, LOW  }, // 17
	{111 ms,  ROTX, LOW  }, // 18
	STOP_ACTION,
};

// Sequenz zum Auslsen im oder Beenden des LV. Wird passend fortgesetzt, wenn Coil 2 abschaltet.
// Das Timing ist schneller als beim normalen Auslsen, das fr LeaveLV zu Error30 
//  fhren wrde und auch nicht ausreichend angepasst werden kann.
// 
// Das normale Timing wird fr Auslsen im LV akzeptiert, wenn OTP2 Lo gesetzt wird.
//  -> Wie ist das Coil2-Timing dann???
//  Spter noch Hi an OPT2 gibt fr beide Flle Fehler!
//
sAction c1_L_StartInLV[] = {
	{ 0 ms,  OPT1, LOW  },
	//unntig:
	{ 0 ms,  OPT2, HIGH },	// (OPT2 bleibt Hi)
//{ 0 ms,  OPT2, LOW  },
	{ 0 ms,  ROT1, HIGH }, // unntig: R1 ist schon Hi

	{16 ms,  ROTX, HIGH }, // statt 14, das regulr folgende LOW,HIGH entfllt
	{16 ms,  ROT2, LOW  }, // statt 32 im Nicht-LV

	{34 ms,  ROT2, HIGH }, // statt 48 im Nicht-LV
	{ 0 ms,  ROT3, LOW  },
	// Frheres/spteres ROT2_H, ROT3_L verschiebt C2_H im LeaveLV entsprechend!!!
	// C2_H im ShootLV bleibt bei ca. 51, aber:
	/* Wenn hier OPT1_H_L_H so eingefgt wird, lst das C2_H im ShootLV schon bei 45 ms aus,
	//  das damit an die 43 ms des C2_H in LeaveLV heranrckt! Die Limits mssen dann sehr eng gesetzt werden!
	{ 43 ms,  OPT1, HIGH},	// (Versuch, LeaveLV ohne Coil 2 korrekt zu timen)
	{ 44 ms,  OPT1, LOW},
	{ 46 ms,  OPT1, HIGH},
	//*/
		HALT_ACTION,
};


// Nachdem der LV-Start am Coil-2 Hi Timing erkannt wurde, mssen/knnen(?) 
//  die ROT-Pegel noch korrigiert werden:
//
sAction c2_H_EnterLV[] = {
	{0 ms,    ROTX, LOW  }, // war schon zu weit! Wieder zurck
	{0 ms,    ROT3, HIGH }, // ROT3 auch nicht
	{0 ms,    ROT2, HIGH }, // (sollte schon stimmen)
	{6 ms,    OPT2, HIGH },	//???? -> Curtain ffnet und nach 6 ms geht OPT2 Hi (Htte gar nicht low werden sollen!)
	STOP_ACTION,						// dieser Stand wird dann eingefroren (der innere Curtain hlt auch nachdem Coil 1 lst, das OPT1 Signal wird nur Hi, weil die LED aus geht)
};

sAction c1_H_EnterLV[] = {
	{0 ms,    ROTX, LOW  }, // war schon zu weit! Wieder zurck
	{0 ms,    ROT3, HIGH }, // ROT3 auch nicht
	{0 ms,    ROT2, HIGH }, // (sollte schon stimmen)
	{6 ms,    OPT2, HIGH },	// ???? -> Curtain ffnet und nach 6 ms geht OPT2 Hi (Htte gar nicht low werden sollen!)

//{111 ms,  OPT1, LOW},	// (Ist bei leaveLV schon vor dem Start Lo: Erscheint nur wg. fehlender Coil-Pwr HIGH!?)

	STOP_ACTION,						// dieser Stand wird dann eingefroren (der innere Curtain hlt auch nachdem Coil 1 lst, das OPT1 Signal wird nur Hi, weil die LED aus geht)
};

// Nachdem das LV-Ende am Coil-2 Hi Timing erkannt wurde, mssen/knnen(?) 
//  die ROT-Pegel noch korrigiert werden, die hier frher kommen mssen,
//  aber noch auf dem Stand der reg. Auslsung nach 43 ms sind:
//
sAction c2_H_LeaveLV[] = {
	{0 ms,    ROT2, HIGH }, // ROT2 msste schon Hi sein!
	{0 ms,    ROT3, LOW  }, //  und ROT3 Lo
	{4 ms,    OPT1, HIGH },	// -> innerer Curtain schliet nachdem der Motor weiter dreht und Coil 1 (zeitgleich mit 2) lst und nach 6 ms geht OPT1 Hi
	{6 ms,    OPT1, LOW  },
	{7 ms,    OPT1, HIGH },
	{16 ms,   ROT3, HIGH }, // dann der (vorgezogene) Rest der ROT..
	{43 ms,   ROT4, LOW  },
	{82 ms,   ROT4, HIGH },
	{82 ms,   ROT1, LOW  },
	{98 ms,   ROTX, LOW  },
	STOP_ACTION,						// fertig
};
#define c1_H_LeaveLV	c2_H_LeaveLV		// ohne Coil2: fr Coil1 identisch

// Fortsetzung des Timings bei Auslsung im LV 
//
//#if defined(TRIGGER2_MODE)	// Mit Coil 2
sAction c2_H_FinishLVShoot[] = {			// Coil 2 wird inaktiv
	{6 ms,    OPT2, HIGH }, 				// -> Curtain ffnet und nach 6 ms geht OPT2 Hi; sollte schon Hi sein, wird bentigt!!!!
	LONGWAIT_ACTION,					// hier warten, bis Coil 1 Hi wird und den Ablauf fortsetzt
// #else
// sAction c1_H_FinishLVShoot[] = {
// #endif	
#define c1_H_FinishLVShoot (c2_H_FinishLVShoot + 2)	
	// Coil 1 wird inaktiv: fr Coil 1 allein kein LONGWAIT (mehr), weil bei HALT in c1_L_Start
	//	gewartet wurde
	{0 ms,    OPT2, HIGH }, 				// falls berrollt
	{4 ms,    OPT1, HIGH }, // 11 ohne Offset 78
	{5 ms,    OPT1, LOW  }, // 12
	{7 ms,    OPT1, HIGH }, // 13
	{18 ms,   ROT3, HIGH }, // 14
	{45 ms,   ROT4, LOW  }, // 15
	{90 ms,   ROT4, HIGH }, // 16
	{90 ms,   ROT1, LOW  }, // 17
	{108 ms,   ROTX, LOW  }, // 18
	STOP_ACTION,
};

// Nchste sAction Liste auswhlen fr die Fortsetzung eines Coil-Start Ereignisses,
//  das jetzt nach dt Millisekunden durch ein Coil-Ende Signal abgeschlossen wurde.
//
inline sAction* SelectFinishAction(unsigned long dt)
{
  // Coil-2 Timing der 700D:
  // - Das regulre C2_Hi kommt bei 68 ms, im LV nach 50 ms
  // - Beim LV-Start kommt C2_Hi bei 73 ms (Verzgerung durch das Anhalten des Motors?!)
  // - Am Ende des LV ist der C2 Puls nur 43 ms breit
  //
  //BlinkLED(dt);//45 ms fr Shoot!
  if (dt < HiLIMIT_C2_LEAVE_LV) {		// LV-Stop
    return c2_H_LeaveLV;
  }
  else if (IsInWindow_LVShoot(dt)) {		// Auslsen im LV
    return c2_H_FinishLVShoot;
  }
  else if (dt >= LoLIMIT_C2_ENTER_LV) {	// EnterLV
    return c2_H_EnterLV;
  }
  else {																	// (700D 68 ms, 600D 77 ms:) nicht-LV-Shoot
    return c2_H_FinishShoot;
  }
    /*/ // Variante, die (auch) die aktuelle Liste mit auswertet:
    if (actionPtr == c1_L_Start) {		// (noch) nicht im LV?
	    if (dt < 70 ms) {		// ca. 68 ms: regulres Auslsen
		    actionPtr = c2_H_FinishShoot;
	    }
	    else {										// ca. 73 ms: LV-Start oder Restart
		    actionPtr = c2_H_EnterLV;
	    }
    }
    else {
	    if (digitalRead(TRIGGER1) == TRIGGER1_ACTIVE) {		// im LV: Coil 1 noch aktiv?
		    actionPtr = c2_H_FinishLVShoot; 					//  dann ist es Shoot
	    }
	    else {
		    actionPtr = c2_H_LeaveLV;					// beim Beenden des LV kommt Coil 1 sofort zurck
	    }
    }
    //*/
}

