//#ifdef EOS_M
// 
// Der Shutter der EOS M ist im Ruhezustand offen und selbsthaltend. 
// Zu Beginn einer Aufnahme werden der Motor und die Coil aktiviert, um den Shutter dadurch
//  scharf zu machen, dass er ber den selbsthaltenden Offen-Punkt hinaus bewegt und bis
//  zu derer Deaktivierung von der Coil offen gehalten wird. LS1 prft dabei, wann die
//  Mechanik 'aufgezogen' ist und der Motor wieder abgeschaltet werden muss.
// Zum Ende der Aufnahme wird die Coil abgeschaltet und LS2 prft das Schlieen der Blades.
// Danach ist die Aufnahme abgeschlossen und der Verschluss wird wieder geffnet. Dabei stoppt
//  der Motor sobald der selbsthaltende Offen-Punkt erreicht ist, was LS1=H anzeigt.
// Das Timing (in Millisekunden) ist fr alle Belichtungen gleichartig (LS1==OPT1, LS2==OPT2): 
// 
//  (1.) 0    M1: H
//       5    Coil: L
//       7    LS1: L        (M1 und Coil beide geschaltet!)
//      21    LS1: H,  M1: L
//    (2.)    T+x    Coil: H
//        5    LS2: L
//        7    LS2: H
//        8    LS2: L
//       16    LS2: H
//    (3.) 70+T  M1:H
//          0    LS2:L
//         13        LS1:L
//         21    LS2:H
//         27    LS2:L
//         34    LS2:H
//         42        LS1:H
// 
// Es gibt drei Referenz-Zeitpunkte:
// 1.) Bei t=0 startet der Motor. Da dies auch bei (3) passiert, muss gewartet werden, ob Coil LOW folgt.
//     Die folgenden LS1 Signale knnen dann auf Coil LOW bezogen werden.
// 2.) Das Ende der Aufnahme wird an der Abschaltung der Coil erkannt und die LS2-Signale werden generiert.
// 3.) Der Verschluss wird wieder geffnet. Dazu luft der Motor an, ohne dass die Coil aktiviert wird.
//     Bei kurzer Belichtung knnte diese Sequenz auch als Fortsetzung von 2.) implementiert werden,
//       im Bulb-Mode bleibt der Shutter aber noch fr bis zu 6 Sekunden geschlossen - auch wenn
//      "Rauschreduzierung bei Langzeitaufnahmen" abgeschaltet ist.
//     Das zeitgleich erfolgende LS2:L kann um 5 ms verzgert werden, bis klar ist, ob Coil:L folgt oder nicht.
// 

#define COIL_ACTIVE   LOW
#define COIL_TRIGGER  CHANGE
#define MOTOR_ACTIVE  HIGH
#define MOTOR_TRIGGER RISING

//#define MOTOR_GATE  // Motor-Signal nur nach Auslsung akzeptieren: UNVOLLSTNDIG!

// Startwerte:
#define OPT2_SUP HIGH
#define OPT1_SUP HIGH

sAction coilStartAction[] = {      // Coil aktiv (kommt ca. 4 ms nach Motor!)
  { 0 ms, OPT2, HIGH},    // Motor wurde zuerst aktiv und hat evtl. OPT2 LOW gesetzt: Jetzt korrigieren!
  { 3 ms, OPT1, LOW},//2
  {17 ms, OPT1, HIGH},//16
  STOP_ACTION,
};

sAction coilStopAction[] = {      // Coil inaktiv
  { 4 ms, OPT2, LOW},
  { 7 ms, OPT2, HIGH},
  { 8 ms, OPT2, LOW},
  {15 ms, OPT2, HIGH},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

#ifdef MOTOR_GATE
#define MOTOR_DELAY 0
#else
#define MOTOR_DELAY 5
#endif

 sAction motorAction[] = {      // Motor an: Sequenz fr das Aufziehen nach der Aufnahme.
  { MOTOR_DELAY ms, OPT2, LOW},      // kommt im Original sofort, hier 5 ms verzgert, falls auch noch Coil low wird: dann ist Auslse-Start, bei dem der Motor auch aktieviert wird.
  {13 ms, OPT1, LOW},
  {21 ms, OPT2, HIGH},
  {27 ms, OPT2, LOW},
  {34 ms, OPT2, HIGH},
  {42 ms, OPT1, HIGH},
  STOP_ACTION,
};

 //#endif //EOS_M
