//#if defined(EOS_M100)
// 
// Die EOS M10/M100 erwarten High an Opt1. Damit kann der Emulator einfach
//  mit der Spannung der Coil 1 versorgt werden (auch wenn diese spt kommt)
//  und als gewhnlicher Sketch geladen werden.
// 
// Der Shutter der EOS M10/M100 ist im Ruhezustand offen und selbsthaltend. 
// Zu Beginn einer Aufnahme werden der Motor und die Coil1 aktiviert, um den Shutter dadurch
//  scharf zu machen, dass er ber den selbsthaltenden Offen-Punkt hinaus bewegt und bis
//  zu deren Deaktivierung von der Coil1 offen gehalten wird. 
// OPT2 == Hi zeigt an, dass der Shutter offen ist. 
// Die Belichtung beginnt elektronisch, etwa 10 ms nachdem der Motor den Shutter aufgezogen hat,
//  was an OPT1 = Lo erkannt wird.
// 
//  Zum Ende der Aufnahme wird die Coil abgeschaltet und OPT2 zeigt das Schlieen der Blades
//  mit Low-High-Low (und, nachdem die OPT2-Power abgeschaltet wurde, wieder High) an.
//  Danach zieht der Motor den Shutter wieder auf, bis Opt1 ffnet (hi wird).
//  Etwa 130 ms spter schiebt die Coil2 den Shutter wieder auf, was von OPT2
//  mit High-Low-High angezeigt wird.
//  Danach ist die Aufnahme abgeschlossen und der Verschluss wieder selbsthaltend geffnet.
// 
// Anders als bei der M3 wird diese Sequenz auch im Bulb-Mode ausgefhrt.
// Deshalb muss das Motorsignal fr die Emulation nicht ausgewertet werden.
// (Die Pause von ber 2s, die die M100 beim Drcken des Auslsers im Bulb-Mode
//  manchmal macht, hat nichts mit der Emulation zu tun, sie tritt auch mit Shutter auf.)
// 
#define COIL_ACTIVE   LOW
#define COIL_TRIGGER  CHANGE

// Startwerte
#define OPT1_SUP HIGH   // wie M, umgekehrt wie M3
#define OPT2_SUP HIGH

sAction coilStartAction[] = {      // Coil aktiv
  { 0 ms, OPT2, HIGH},
  { 0 ms, OPT1, HIGH},
  {20 ms, OPT1, LOW},     // (stoppt Motor)
  {25 ms, OPT1, LOW},     // warten bis Motor sicher aus, dann Motor-Int. freigeben:
  STOP_ACTION,
};

#define csaO 126    // Delay bis Coil2 aktiviert wird
sAction coilStopAction[] = {      // Coil inaktiv
  { 4 ms, OPT2, LOW},
  { 7 ms, OPT2, HIGH},
  { 8 ms, OPT2, LOW},         // (dann lo lassen, auch wenn OPwr2 aus ist)
  {64 ms, OPT1, HIGH},         // Motor stoppen
  {(csaO+6) ms, OPT2, HIGH},     // Coil2 schiebt den Curtain in die selbsthaltende Position
  {(csaO+9) ms, OPT2, LOW},
  {(csaO+11) ms, OPT2, HIGH},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

//#endif
