//#ifdef EOS_M3
// 
// Die EOS M3 erwartet Low an Opt1. Wenn dies mit einem Pull-Down 
//  sichergestellt wird, kann der Emulator als gewhnlicher Sketch geladen
//  und mit der Spannung der Coil 1 versorgt werden.
//
// Die Shutter der EOS M3++ sind im Ruhezustand offen und selbsthaltend. 
// Zu Beginn einer Aufnahme werden der Motor und die Coil1 aktiviert, um den Shutter dadurch
//  scharf zu machen, dass er ber den selbsthaltenden Offen-Punkt hinaus bewegt und bis
//  zu deren Deaktivierung von der Coil1 offen gehalten wird. 
// OPT2 == Hi zeigt an, dass der Shutter offen ist. 
// Die Belichtung beginnt elektronisch, etwa 10 ms nachdem der Motor den Shutter aufgezogen hat,
//  was an OPT1 = High erkannt wird.
// 
// a) Festzeit-Belichtung:
//  Zum Ende der Aufnahme wird die Coil abgeschaltet und OPT2 zeigt das Schlieen der Blades
//  mit Low-High-Low (und, nachdem die OPT2-Power abgeschaltet wurde, wieder High) an.
//  Danach zieht der Motor den Shutter wieder auf, bis Opt1 ffnet (low wird).
//  (OPT1 wird dann sofort wieder Low, vmtl. weil die OPT1-Pwr abgeschaltet wird.)
//  Etwa 130 ms spter schiebt die Coil2 den Shutter wieder auf, was von OPT2
//  mit High-Low-High angezeigt wird.
//  Danach ist die Aufnahme abgeschlossen und der Verschluss wieder selbsthaltend geffnet.
//  Das Timing (in Millisekunden) ist fr alle Belichtungszeiten gleichartig.
// 
// b) Bulb-Mode:
//  Hier luft der Motor schon an, bevor Coil1 lst. Nach OPT1 = Low luft der Motor noch
//  weiter, bis OPT1 wieder High wird. Erst jetzt lst Coil1 und OPT2 zeigt mit Low-High-Low
//  an, dass der Sutter schliet. Nach einer Pause von ca. 280 ms startet dann das 
//  Wiederaufziehen des Shutters wie bei der Festzeit-Belichtung, wobei die Pause vor der
//  Coil2-Aktion ca. 200 statt 120 ms dauert und von der Lnge her schwankt.
//  Deshalb muss im Bulb-Mode der Motor-Interrupt verwendet werden:
//  1.) Gleich nach Beginn der Aufnahme (nach Opt1_H) wird er durch das in die
//      coilStartAction Tabelle eingefgte MOTOR_ACTION Makro aktiviert. Hinter diesem 
//      folgt als Wartepunkt M_WAIT ein (unsinniger) Befehl, der zeitlich nicht erreicht werden kann.
//      Wenn der Motor-Interrupt vor C1_H auslst, luft eine Bulb-Aufnahme und das Timing
//      wird angepasst, indem das angepasste Timing in coilStopBulbAction gewhlt wird.
//      Wenn kein Motor-Interrupt vor C1_H ausgelst wird, was daran erkannt wird, dass die
//      Ausfhrung bei M_WAIT hngt, wird der Motor-Interrupt wieder deaktiviert.
//  2.) Nachdem C1_H die Abarbeitung von coilStopBulbAction angestoen hat, muss, sobald die 
//      Opt2 H-L-H Sequenz erledigt ist, wieder auf den Motor gewartet werden, weil die
//      Wartezeit nicht fest ist. Dazu folgt in coilStopBulbAction jetzt ein MOTOR_ACTION Makro,
//      gefolgt vom Wartepunkt CS_WAIT. An dem erkennt die Ablauflogik, ob der Motor_Interrupt 
//      von hier aktiviert wurde oder in der coilStartAction. (CS_WAIT ist eigentlich unntig,
//      auch auf M_WAIT geprft werden knnte.)
//      
// Bei der M3++ ist das Motorsignal gepulst. Der Interrupt muss deshalb nach 
//  Auftreten deaktiviert werden!
// Der Motor-Interrupt lst sofort nach Aktivierung aus, auch wenn das Motorsignal Low ist!
//  Es ist unklar warum. Zur Unterdrckung wird deshalb erst der zweite Motor-Interrupt 
//  ausgewertet, was kein Problem ist, da das Motorsignal gepulst ist.
// 
// Die Wartezeit bis zum abschlieenden Einrasten des Shutters durch Coil2 ist fr Bulb
//  ebenfalls verschieden, aber konstant, so dass sie in coilFinishBulbAction fest kodiert 
//  werden kann.
// 
#define COIL_ACTIVE   LOW
#define COIL_TRIGGER  CHANGE
#define MOTOR_TRIGGER RISING

//#define MOTOR_GATE  // Motor-Signal nur nach Auslsung akzeptieren: UNVOLLSTNDIG!

// Startwerte
#define OPT1_SUP LOW
#define OPT2_SUP HIGH

sAction coilStartAction[] = {      // Coil aktiv
  { 0 ms, OPT2, HIGH},    //
  { 0 ms, OPT1, LOW},
  {22 ms, OPT1, HIGH},    // (stoppt Motor)
  {25 ms, OPT1, HIGH},    // warten bis Motor sicher aus, dann Motor-Int. freigeben:
  MOTOR_ACTION,
  STOP_ACTION,
};

#define csaO 185
sAction coilStopAction[] = {      // Coil inaktiv
  { 4 ms, OPT2, LOW},
  { 7 ms, OPT2, HIGH},
  { 8 ms, OPT2, LOW},         // (dann lo lassen, auch wenn OPwr2 aus ist)
  {64 ms, OPT1, LOW},         // Motor stoppen
  //{(csaO) ms, OPT2, LOW},     // Coil2 schiebt den Curtain in die selbsthaltende Position
  {(csaO+6) ms, OPT2, HIGH},
  {(csaO+9) ms, OPT2, LOW},
  {(csaO+11) ms, OPT2, HIGH},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

sAction motorAction[] = {      // Motor startet whrend Belichtung (nur im Bulb-Mode!)
  {58 ms, OPT1, LOW},           // fast aufgezogen
  {80 ms, OPT1, HIGH},          // aufgezogen, lst C1_L aus
  // jetzt warten auf C1_L: noch kein STOP_ACTION, sonst geht actionPtr verloren
  //  denn an actionPtr==motorAction+2 wird Bulb erkannt (also Motor vor c1_H)!
#define M_WAIT (motorAction + 2)
  {999 ms, OPT1, HIGH},          
  STOP_ACTION,
};

sAction coilStopBulbAction[] = {      // Coil inaktiv fr Bulb
  { 4 ms, OPT2, LOW},
  { 7 ms, OPT2, HIGH},
  { 8 ms, OPT2, LOW},
  MOTOR_ACTION,
#define CS_WAIT (coilStopBulbAction + 4)
  {8888 ms, OPT2, LOW},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

sAction coilFinishBulbAction[] = {      // Motor startet im Bulb-Mode zum zweiten Mal (nach der Belichtung)
  {55 ms, OPT1, LOW},           // Motor hat Ziel erreicht
  {(55+202) ms, OPT2, HIGH},    // Coil2 starten und rastet Shutter ein
  {(55+202+3) ms, OPT2, LOW},
  {(55+202+5) ms, OPT2, HIGH},
  BLINK_ACTION,        // wenn DEBUG_BLINK defined
  STOP_ACTION,
};

//#endif //EOS_M3
