//#ifdef EOS_M50
// 
// Die EOS M50 erwartet Low an OPTM. Wenn dies mit einem Pull-Down 
//  sichergestellt wird, kann der Emulator als gewhnlicher Sketch geladen
//  und mit der Spannung der Coil 1 versorgt werden.
//
// Beide Blades des EOS M50 Shutters sind im Ruhezustand offen und selbsthaltend. 
// Zu Beginn einer Aufnahme werden der Motor und die Coil 1 aktiviert, um den Shutter dadurch
//  scharf zu machen, dass Blade 1 ber den selbsthaltenden Offen-Punkt hinaus bewegt und bis
//  zu deren Deaktivierung von der Coil 1 offen gehalten wird. 
// OPT1 == Hi zeigt an, dass der Shutter offen ist. 
// Die Belichtung beginnt elektronisch, knapp 20 ms nachdem der Motor den Shutter
//  aufgezogen hat, was anhand der Motor-Lichtschranke OPTM geprft wird.
// 
// a) Festzeit-Belichtung:
//  Zum Ende der Aufnahme wird die Coil1 abgeschaltet und OPT1 zeigt das Schlieen von Blade1
//  mit Low-High-Low-High an.
//  Anschlieend (nach ca. 13ms) zieht der Motor den Shutter wieder auf:
//    Blade 2 wird zugezogen, LS2 ffnet zu Beginn (wird low).
//    Mit geringem Versatz beginnt Blade 1 zu ffnen, LS1 zeigt Hi-Lo-Hi.
//    Wenn beide angekommen (vorgespannt) sind, sind beide Coils in Kontakt,
//     der Zustand hlt zunchst aber auch selbst.
//    Trotzdem wird dabei Coil2 aktiviert:
//     Ausgelst durch LS2=Lo beginnt Coil2 zu halten (nach 38ms), whrend der Motor anhlt
//     (gesteuert durch LSm) und LS2 wieder Hi wird  wg. LS-Pwr=aus.
//     Coil2  lst nach 70ms wieder, zugleich wird LS2 Lo, weil die LS-Pwr
//      wieder eingeschaltet wird.
//     Dadurch ffnet Blade 2 wieder (fllt auf) und LS2 wird wieder Hi.
// 
// b) Im Bulb-Mode luft der Motor erst nach 340 ms wieder an, das muss abgewartet werden!
//  Die Pulse auf LS2 bei 26 ms und bei C1_H werden nicht bentigt. Zumindest der
//  Puls bei 26 ms strt aber auch nicht.
// 
// Auf die Coil2-Signale kann nicht verzichtet werden, weil das zugehrige Timing 
//  nicht genau genug auf den Motor bezogen werden kann (wg. Unterschieden im 
//  Timing fr versch. Belichtungszeiten).
// 
#define COIL_ACTIVE   LOW
#define COIL_TRIGGER  CHANGE

// Startwerte
#define OPT1_SUP HIGH
#define OPT2_SUP HIGH
#define OPTM_SUP LOW

sAction coilStartAction[] = {      // Coil1 aktiv
  
  // Der Motor startet und schiebt Blade 1 ber den Selbsthaltepunkt
  //  hinaus, hlt ihn aber mit Coil 1. Die Motor-Lichtschranke zeigt
  //  dies mit Hi-Lo-Hi an.
  //
  {15 ms, OPTM, HIGH},    
  {23 ms, OPTM, LOW},
  //**{26 ms, OPT2, LOW},   //** das Bulb-Protokoll zeigt einen OPT2-Puls, der aber nicht bentigt wird und auch nicht strt
  {27 ms, OPTM, HIGH},
  //**{34 ms, OPT2, HIGH},  //** (nur Bulb, s.o.)
  STOP_ACTION,
};

sAction coilStopAction[] = {      // Coil1 wird inaktiv: Ende Belichtung
  { 1 ms, OPT1, LOW},             // Blade 1 fllt zu, LS1 zeigt lo-hi-lo-hi
  { 3 ms, OPT1, HIGH},
  { 4 ms, OPT1, LOW},
  { 7 ms, OPT1, HIGH},            // (wird bentigt!)
  MOTOR_ACTION,                   // auf den Motor warten
  STOP_ACTION,
};

sAction motorAction[] = {         // Motor startet nach der Belichtung, im Bulb-Mode erst spt
#define mot 0                     // hier startet der Motor wieder
  {(mot+ 0) ms, OPT1, LOW},       // zeitgleich zu Motorstart(!), wird aber bentigt
  {(mot+ 4) ms, OPTM, LOW},
  {(mot+13) ms, OPTM, HIGH},
  {(mot+13) ms, OPT2, LOW},
  {(mot+16) ms, OPT1, HIGH},
  {(mot+22) ms, OPT1, LOW},       // (wird nicht kontrolliert)
  {(mot+30) ms, OPT1, HIGH},      // (wird nicht kontrolliert)
  {(mot+38) ms, OPTM, LOW},       // lst C2_L aus
  STOP_ACTION,
};

sAction coil2StartAction[] = {      // Coil2 aktiv
//{ 5 ms, OPTM, HIGH},              // entsteht vmtl. durch Abschalten der OPTM-Pwr, muss nicht emuliert werden
//{14 ms, OPT2, HIGH},              // entsteht durch Abschalten der OPT2-Pwr, muss nicht emuliert werden
//{22 ms, OPTM, LOW},               // (nur falls zuvor abgeschaltet wurde)
  
  STOP_ACTION,
};

sAction coil2StopAction[] = {      // Coil2 wird inaktiv
//{ 0 ms, OPT2, LOW},              // entsteht durch Wiedereinschalten der OPT2-Pwr, s.o.
  { 9 ms, OPT2, HIGH},             // 1 ms ist auch ok, 15 ist zu viel
  STOP_ACTION,
};

//#endif //EOS_M50
